/*
 * This file is in the Public Domain
 */
package com.ldodds.foaf.jena;

import java.io.*;
import java.net.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Iterator;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;
import com.hp.hpl.jena.vocabulary.DC;
import com.hp.hpl.jena.vocabulary.RSS;

import com.ldodds.foaf.FOAFWriter;
import com.ldodds.foaf.model.Person;

/**
 * TODO -- description of FOAFWriterImpl
 * @author ldodds
 */
public class FOAFWriterImpl implements FOAFWriter
{
    public static final String ADMIN_URI = "http://webns.net/mvcb/";
    public static final String ADMIN_AGENT = "generatorAgent";
    public static final String ADMIN_ERRORS = "errorReportsTo";
    public static final String API_URI = "http://www.ldodds.com/foaf/foaf-beans/";
    public static final String CONTACT_URI = "mailto:foaf-beans@ldodds.com";
    
    private OutputStream _out;
    private String _baseURI;
    private URL _toPostTo;
    private Model _model;
    private Resource _primaryTopic;
    
    public FOAFWriterImpl(Model model, String baseURI, File file) throws FileNotFoundException
    {
        _baseURI = baseURI;
        _out = new FileOutputStream(file);
        _model = model;
    }
    
    public FOAFWriterImpl(Model model, String baseURI, String filename) throws FileNotFoundException
    {
        _baseURI = baseURI;
        _out = new FileOutputStream(filename);
        _model = model;
    }
    
    public FOAFWriterImpl(Model model, String baseURI, OutputStream out)
    {
        _baseURI = baseURI;
        _out = out;
        _model = model;
    }
    
    public FOAFWriterImpl(Model model, String baseURI, URL toPostTo)
    {
        _baseURI = baseURI;
        _toPostTo = toPostTo;
        _model = model;
    }
    
    /** 
     * @see com.ldodds.foaf.FOAFWriter#writePerson(com.ldodds.foaf.model.Person)
     */
    public void writePerson(Person person, boolean isPrimaryTopic)
    {
        PersonTripleBuilder builder = new PersonTripleBuilder(_model, true);
        Resource resource = builder.addPerson(person);
        for (Iterator iter = person.getFriends().iterator(); iter.hasNext(); )
        {
            builder.addFriend(resource, (Person)iter.next());
        }
        if (isPrimaryTopic)
        {
            _primaryTopic = resource;
        }

    }

    /** 
     * @see com.ldodds.foaf.FOAFWriter#save()
     * @param asPPD generate foaf:PersonalProfileDocument, or just the triples
     */
    public void save(boolean asPPD) throws IOException
    {
        if (asPPD)
        {
            generateAdditionalMetadata();
        }
        try
        {
	        if (_out != null)
	        {            
	            getWriter().write(_model, _out, _baseURI);
	            return;
	        }
	        
	        postToURL();
	        
        } finally
        {
            try { if (_out != null) _out.close(); } catch (Exception e ) {}
        }
    }

    /**
     * TODO we really ought to be doing a PUT here. We'll go with POST for now.
     */
    private void postToURL() throws IOException
    {
        HttpURLConnection connection = (HttpURLConnection)_toPostTo.openConnection();
        connection.setDoOutput(true);
        connection.setAllowUserInteraction(false);
        connection.setUseCaches(false);
        _out = connection.getOutputStream();
        getWriter().write(_model, _out, _baseURI);
        _out.close();
        //TODO checking of response codes...
    }

    /**
     * 
     */
    private void generateAdditionalMetadata()
    {
        Resource theFile = _model.createResource(_baseURI);
        theFile.addProperty(RDF.type, FOAF.PersonalProfileDocument);
        theFile.addProperty( FOAF.primaryTopic, _primaryTopic);
        
        //TODO allow these to be configured
        Resource thisApp = _model.createResource(API_URI);        
        Resource mailto = _model.createResource(CONTACT_URI);
        
        theFile.addProperty( _model.createProperty(ADMIN_URI, ADMIN_AGENT), thisApp);
        theFile.addProperty( _model.createProperty(ADMIN_URI, ADMIN_ERRORS), mailto);
                
        if (_primaryTopic.hasProperty(FOAF.name))
        {
            theFile.addProperty( DC.title, "FOAF Description for " 
                                                       + _primaryTopic.getProperty(FOAF.name).getString());
            theFile.addProperty( DC.description, "Friend of a Friend (FOAF) description, " +
                                                                   "containing personal metadata for " + 
                                                                  _primaryTopic.getProperty(FOAF.name).getString());
        }                                                                    
                                                                                                                                       
        Date now = Calendar.getInstance().getTime();
        //TODO extend to use correct W3C Date Time Format for hrs/mins/secs
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
        theFile.addProperty( DC.date, fmt.format(now));        
    }

    private RDFWriter getWriter()
    {
        RDFWriter writer = _model.getWriter("RDF/XML-ABBREV");
        configurePrefixMappings(writer);
        return _model.getWriter();
    }
    
    private void configurePrefixMappings(RDFWriter writer)
    {
        _model.getGraph().getPrefixMapping().setNsPrefix("foaf", FOAF.getURI());
        _model.getGraph().getPrefixMapping().setNsPrefix("dc", DC.getURI());
        _model.getGraph().getPrefixMapping().setNsPrefix("rss", RSS.getURI());   
        
        //FIXME why am I still getting no xml decl and rel. uris
        writer.setProperty("relativeURIs", "");
        writer.setProperty("showXmlDeclaration", Boolean.TRUE);
        writer.setProperty("blockRules", new Resource[] {RDFSyntax.propertyAttr, RDFSyntax.parseTypeResourcePropertyElt});
        writer.setProperty("prettyTypes", new Resource[] { FOAF.Person });
    }
}
